@@ -2,10 +2,10 @@
 						buildinglayer.h  -  description
 							-------------------
 	begin                : september 20th, 2003
-	copyright            : (C) 2003-2007 by Duong-Khang NGUYEN
-	email                : neoneurone @ users sourceforge net
+	copyright            : (C) 2003-2008 by Duong Khang NGUYEN
+	email                : neoneurone @ gmail com
 
-	$Id: buildinglayer.h 223 2007-05-07 15:00:53Z neoneurone $
+	$Id: buildinglayer.h 375 2008-10-28 14:47:15Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -212,7 +212,7 @@ linear index of a structure is calculated by this formula:
 //========================================================================
 /** see "layer.h" for more informations
 */
-	GUIContainer*
+	GUIContainer* const
 	QueryStructure(
 		const uint & w,
 		const uint & l ) const;
