@@ -4,14 +4,14 @@
 	project              : OpenCity
 	codename             : Delphine
 	begin                : may 28th, 2003
-	copyright            : (C) 2003-2007 by Duong-Khang NGUYEN
-	email                : neoneurone @ users sourceforge net
+	copyright            : (C) 2003-2008 by Duong Khang NGUYEN
+	email                : neoneurone @ gmail com
 
 	details              :
 			This is a game project which targets the ultimate city
 		simulation just like like xxxCity or something like that :)
 
-	$Id: main.cpp 304 2007-12-22 21:08:40Z neoneurone $
+	$Id: main.cpp 383 2008-11-03 21:36:04Z neoneurone $
  ***************************************************************************/
 
 /***************************************************************************
@@ -41,8 +41,8 @@
 	AudioManager, Conf, ModelLoader, and MapGen::MapMaker without major
 	modifications.
 
-	\author Duong-Khang NGUYEN (email: neoneurone [at] users [dot] sf [dot] net)
-	\author Frdric RODRIGO
+	\author Duong-Khang NGUYEN (email: duongkhang.nguyen gmail.com)
+	\author Frédéric RODRIGO
 */
 
 // Useful enumerations
@@ -73,6 +73,7 @@ extern GlobalVar gVars;
 #include <ctime>				// For time
 
 // Test XPath
+#include "property.h"
 #include "propertymanager2.h"
 
 
@@ -83,6 +84,7 @@ extern GlobalVar gVars;
 	#include <sys/stat.h>		// mkdir
 #else
 // Win32 specifics
+	#include <windows.h>		// MessageBox
 	#include <shlobj.h>			// Windows shell technologies
 	#define DATADIR "C:/Program Files"
 	#define SYSCONFDIR DATADIR
@@ -93,30 +95,30 @@ extern GlobalVar gVars;
 	#define OC_WINDOW_POS_Y			20
 	#define OC_WINDOW_WIDTH			750
 	#define OC_WINDOW_HEIGHT		560
-	#define OC_WINDOW_BPP_DEFAULT	32				// OC uses this by default
-	#define OC_WINDOW_BPP_16		16
+	#define OC_WINDOW_BPP_DEFAULT	0			///< Use the current video bpp
 	#define OC_FULLSCREEN_WIDTH		1024
 	#define OC_FULLSCREEN_HEIGHT	768
 
 // Exit code. This is not an exhaustive list.
-	#define OC_CONFIG_NOT_FOUND			-1
-	#define OC_CONFIG_PARSE_ERROR		-2
-	#define OC_ARGUMENT_ERROR			-3
+	#define OC_ERROR_NOT_FOUND			-1		///< Config files not found
+	#define OC_ERROR_PARSE_CONFIG		-2		///< Config parsing error
+	#define OC_ERROR_ARGUMENT			-3		///< Command line argument error
 
-	#define OC_OUT_OF_MEMORY			-10
+	#define OC_ERROR_MEMORY				-10		///< Out of memory
 
-	#define OC_SDL_INIT_ERROR			-20
-	#define OC_SDL_BPP_ERROR			-21
-	#define OC_SDL_DOUBLEBUFFER_ERROR	-22
-	#define OC_SDL_VIDEORESIZE_ERROR	-23
-	#define OC_SDL_FULLSCREEN_ERROR		-24
+	#define OC_ERROR_SDL_INIT			-20		///< SDL initialization failed
+	#define OC_ERROR_SDL_DOUBLEBUFFER	-21		///< Video double buffer unsupported
+	#define OC_ERROR_SDL_ACCELERATED	-22		///< Accelerated display unsupported
+	#define OC_ERROR_SDL_VIDEORESIZE	-23		///< Unable to resize the window
+	#define OC_ERROR_SDL_FULLSCREEN		-24		///< Unable to go fullscreen
+	#define OC_ERROR_SDL_OPENGL			-25		///< Unable to load OpenGL library dynamically
 
 // Settings file
 	#define OC_CONFIG_FILE_FILENAME		"config/opencity.xml"
 
 // Others macros
 	#define OC_WINDOW_NAME PACKAGE VERSION
-	#define OC_PROGRAM_NAME			"OpenCity standalone/client application"
+	#define OC_PROGRAM_NAME				"OpenCity standalone/client application"
 
 
 
@@ -131,18 +133,22 @@ extern GlobalVar gVars;
 	static bool bRestart		= false;
 
 /// Flags we will pass into SDL_SetVideoMode.
-	static int flags			= 0;
+	static int iVideoFlag		= SDL_OPENGL;
 
 /// The paths are static so that the others can not access this
 	static string sDataDir		= "";
 	static string sSaveDir		= "";
 	static string sConfigDir	= "";
 
+/// Set to true if OpenGL version string is requested from command line
+	static bool bGLVersion		= false;
+
 
    /*=====================================================================*/
 void ocQuit( const int quit_code )
 {
-	cout << "Quit requested, quit code is : " << quit_code
+	cout << endl
+		 << "Quit requested, quit code is : " << quit_code
 	     << endl
 	     << "Bye bye !"
 	     << endl;
@@ -200,9 +206,9 @@ void ocResize( const SDL_ResizeEvent& rcsResizeEvent)
 // Set the new window's size
 	if( SDL_SetVideoMode(
 		rcsResizeEvent.w, rcsResizeEvent.h,
-		gVars.guiVideoBpp, flags ) == 0 ) {
+		gVars.guiVideoBpp, iVideoFlag ) == 0 ) {
 		OPENCITY_FATAL( "Video mode reset failed: " << SDL_GetError( ) );
-		exit( OC_SDL_VIDEORESIZE_ERROR );
+		exit( OC_ERROR_SDL_VIDEORESIZE );
 	}
 	gVars.gpVideoSrf = SDL_GetVideoSurface();
 #endif
@@ -274,7 +280,7 @@ void ocProcessSDLEvents( void )
 
 		case SDL_QUIT:
 		// Handle quit requests (like Ctrl-c).
-			cout << "Quit requested, stoping " << OC_PROGRAM_NAME << "..." << endl;
+			cout << endl << "Quit requested, stoping " << OC_PROGRAM_NAME << "..." << endl;
 			bQuit = true;
 			break;
 		}
@@ -283,45 +289,7 @@ void ocProcessSDLEvents( void )
 
 
    /*=====================================================================*/
-void getFullScreenResolution(uint & w, uint & h)
-{
-	SDL_Rect **modes;
-	int i;
-	
-// Get available fullscreen/hardware modes
-	modes = SDL_ListModes(NULL, flags);
-	
-// Check if there are any modes available
-	if(modes == (SDL_Rect **)0) {
-		OPENCITY_FATAL( "No fullscreen mode available !" );
-		exit( OC_SDL_FULLSCREEN_ERROR );
-	}
-	
-// Check if our resolution is restricted
-	if(modes == (SDL_Rect **)-1) {
-	// Use the default fullscreen size
-		OPENCITY_INFO( "All fullscreen resolutions available. ");
-		w = OC_FULLSCREEN_WIDTH;
-		h = OC_FULLSCREEN_HEIGHT;
-	}
-	else {
-	// Print valid modes
-		//printf("Available Modes\n");
-		w = 0; h = 0;
-		for (i = 0; modes[i]; ++i) {
-			//printf("  %d x %d\n", modes[i]->w, modes[i]->h);
-			if (modes[i]->w > w) {
-				w = modes[i]->w;
-				h = modes[i]->h;
-			}
-		} // for
-		OPENCITY_INFO( "The autodetected resolution of " << w << "x" << h << " pixels is used." );
-	} // else
-}
-
-
-   /*=====================================================================*/
-int initSDL()
+static int initSDL()
 {
 // Initialization of the SDL library
 	OPENCITY_DEBUG( "SDL Initialization" );
@@ -330,75 +298,108 @@ int initSDL()
 	if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
 		// Failed, exit.
 		OPENCITY_FATAL( "SDL video initialization failed: " << SDL_GetError() );
-		return OC_SDL_INIT_ERROR;
+		return OC_ERROR_SDL_INIT;
 	}
 
-// Set the SDL_GL_DoubleBuffer ON for smoother rendering
-// in addition, this is required for the implemented selection method
-	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+// Set the window's caption
+	SDL_WM_SetCaption( PACKAGE " " VERSION, NULL );
+	SDL_WM_SetIcon( IMG_Load(ocDataDirPrefix("graphism/icon/OpenCity32.png").c_str()), NULL );
 
-// Set 8 bits for each color component
-	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 8 );
- 	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
-	flags = SDL_OPENGL | SDL_RESIZABLE | SDL_HWSURFACE;
+// Set the SDL_GL_DOUBLEBUFFER ON for smoother rendering
+	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+	SDL_GL_SetAttribute( SDL_GL_ACCELERATED_VISUAL, 1 );
+
+// Get the OpenGL driver name
+	const char* glDriver;
+	if (gVars.gsOpenGLDriver != "") {
+		OPENCITY_INFO( "Loading OpenGL driver: " << gVars.gsOpenGLDriver );
+		glDriver = gVars.gsOpenGLDriver.c_str();
+	}
+	else {
+		OPENCITY_INFO( "Loading default OpenGL driver..." );
+		glDriver = NULL;
+	}
+
+// Dynamically load the default OpenGL implementation library
+	if (SDL_GL_LoadLibrary(glDriver) < 0) {
+		OPENCITY_FATAL( "Failed to load OpenGL library: " << SDL_GetError() );
+		return OC_ERROR_SDL_OPENGL;
+	}
 
 // Will we go for fullscreen ?
 	if (gVars.gboolFullScreen == true) {
-		flags |= SDL_FULLSCREEN;
-	// IF autodetect THEN
-		if (gVars.guiScreenWidth == 0 or gVars.guiScreenHeight == 0)
-			getFullScreenResolution( gVars.guiScreenWidth, gVars.guiScreenHeight );
+		iVideoFlag |= SDL_FULLSCREEN;
+
+	// Use the current desktop video resolution
+		gVars.guiScreenWidth	= 0;
+		gVars.guiScreenHeight	= 0;
 	}
 	else {
-	// Use the default screen size
-		gVars.guiScreenWidth	= OC_WINDOW_WIDTH;
-		gVars.guiScreenHeight	= OC_WINDOW_HEIGHT;
+		iVideoFlag |= SDL_RESIZABLE;
 	}
 
 // OK, go for the video settings now
-	gVars.gpVideoSrf = SDL_SetVideoMode( gVars.guiScreenWidth, gVars.guiScreenHeight, gVars.guiVideoBpp, flags );
+	gVars.gpVideoSrf = SDL_SetVideoMode( gVars.guiScreenWidth, gVars.guiScreenHeight, gVars.guiVideoBpp, iVideoFlag );
 	if ( gVars.gpVideoSrf == NULL ) {
-	// This could happen for a variety of reasons,
-	// including DISPLAY not being set, the specified
-	// resolution not being available, etc.
-		OPENCITY_ERROR( 
-			"Initialization of 32 bpp video mode failed: " << SDL_GetError() 
+	// This could happen for a variety of reasons, including DISPLAY
+	// not being set, the specified resolution not being available, etc.
+		OPENCITY_ERROR(
+			"Initialization of " << gVars.guiVideoBpp <<
+			" bpp video mode failed: " << SDL_GetError()
 		);
-		OPENCITY_INFO( "Trying 16 bpp... " );
-		gVars.guiVideoBpp = OC_WINDOW_BPP_16;
-
-	// Set 5 bits for each color component
-		SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
-		SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 6 );
-		SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
-		SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 0 );
-		gVars.gpVideoSrf = SDL_SetVideoMode( gVars.guiScreenWidth, gVars.guiScreenHeight, gVars.guiVideoBpp, flags );
-
-		if (gVars.gpVideoSrf == NULL) {
-			OPENCITY_FATAL( "16 bpp mode has failed: " << SDL_GetError() );
-			return OC_SDL_BPP_ERROR;
-		}
-		else {
-			OPENCITY_INFO( "16 bpp works." );
-		}
-	}
-
-//debug cout << "W: " << gVars.gpVideoSrf->w << " /H: " << gVars.gpVideoSrf->h << endl;
-
-
-// Test for DoubleBuffer
-	int iDblBuff = 0;
-	SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &iDblBuff );
-	if ( iDblBuff == 0 ) {
-		OPENCITY_INFO( "Checking video doublebuffer: failed !" );
-		OPENCITY_FATAL( "We need the video doublebuffer." );
-		return OC_SDL_DOUBLEBUFFER_ERROR;
+		return OC_ERROR_SDL_INIT;
 	}
 	else {
-		OPENCITY_INFO( "Checking video doublebuffer: OK !" );
+		OPENCITY_INFO(
+			"Using " << (uint)gVars.gpVideoSrf->w <<
+			"x" << (uint)gVars.gpVideoSrf->h <<
+			" at " << (uint)gVars.gpVideoSrf->format->BitsPerPixel << " bpp"
+		);
+
+	// Store the fullscreen resolution for later use
+		if (gVars.gboolFullScreen == true) {
+			gVars.guiScreenWidth	= (uint)gVars.gpVideoSrf->w;
+			gVars.guiScreenHeight	= (uint)gVars.gpVideoSrf->h;
+		}
+	}
+
+// Retrieve the video driver name
+	#define BUFFER_SIZE 256
+	char myBuffer[BUFFER_SIZE];
+	if (SDL_VideoDriverName(myBuffer, BUFFER_SIZE) != NULL) {
+		OPENCITY_INFO( "Current video driver: " << myBuffer );
+	}
+	else {
+		OPENCITY_ERROR( "Failed to retrieve the video driver name" );
+	}
+
+// Display the OpenGL version string
+	if (bGLVersion == true) {
+		OPENCITY_INFO( "GL vendor: "	 << glGetString( GL_VENDOR ) );
+		OPENCITY_INFO( "GL renderer: "	 << glGetString( GL_RENDERER ) );
+		OPENCITY_INFO( "GL version: "	 << glGetString( GL_VERSION ) );
+		OPENCITY_INFO( "GL extensions: " << glGetString( GL_EXTENSIONS ) );
+	}
+
+// Double buffer available ?
+	int iValue = 0;
+	SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &iValue );
+	if ( iValue == 0 ) {
+		OPENCITY_FATAL( "Video double buffer unsupported" );
+		return OC_ERROR_SDL_DOUBLEBUFFER;
+	}
+	else {
+		OPENCITY_INFO( "Video double buffer supported" );
+	}
+
+// Accelerated display available ?
+	SDL_GL_GetAttribute( SDL_GL_ACCELERATED_VISUAL, &iValue );
+	if ( iValue == 0 ) {
+		OPENCITY_FATAL( "Accelerated visual unsupported" );
+		return OC_ERROR_SDL_ACCELERATED;
+	}
+	else {
+		OPENCITY_INFO( "Accelerated visual supported" );
 	}
 
 	return 0;
@@ -406,8 +407,7 @@ int initSDL()
 
 
    /*=====================================================================*/
-string
-formatPath(const string& rcsPath)
+static string formatPath(const string& rcsPath)
 {
 	string result = rcsPath;
 
@@ -430,15 +430,16 @@ formatPath(const string& rcsPath)
 
 
    /*=====================================================================*/
-void parseArg(int argc, char *argv[])
+static int parseArg(int argc, char *argv[])
 {
 // Command-line options definition
 	enum {
 		OPT_GL_VERSION,
-		OPT_FULLSCREEN,
+		OPT_GL_DRIVER,
+		OPT_FULL_SCREEN,
 		OPT_NO_AUDIO,
-		OPT_DATADIR,
-		OPT_CONFDIR,
+		OPT_DATA_DIR,
+		OPT_CONF_DIR,
 		OPT_GENERATOR_HEIGHT_MAP,
 		OPT_GENERATOR_SEED,
 		OPT_GENERATOR_MAP,
@@ -450,15 +451,17 @@ void parseArg(int argc, char *argv[])
 
 	CSimpleOpt::SOption g_rgOptions[] = {
 		{ OPT_GL_VERSION,				(char*)"--gl-version",				SO_NONE		},
-		{ OPT_FULLSCREEN,				(char*)"--glv",						SO_NONE		},
-		{ OPT_FULLSCREEN,				(char*)"--fullscreen",				SO_NONE		},
-		{ OPT_FULLSCREEN,				(char*)"-fs",						SO_NONE		},
+		{ OPT_GL_VERSION,				(char*)"-glv",						SO_NONE		},
+		{ OPT_GL_DRIVER,				(char*)"--gl-driver",				SO_REQ_SEP	},
+		{ OPT_GL_DRIVER,				(char*)"-gld",						SO_REQ_SEP	},
+		{ OPT_FULL_SCREEN,				(char*)"--full-screen",				SO_NONE		},
+		{ OPT_FULL_SCREEN,				(char*)"-fs",						SO_NONE		},
 		{ OPT_NO_AUDIO,					(char*)"--no-audio",				SO_NONE		},
 		{ OPT_NO_AUDIO,					(char*)"-na",						SO_NONE		},
-		{ OPT_DATADIR,					(char*)"--datadir",					SO_REQ_SEP	},
-		{ OPT_DATADIR,					(char*)"-dd",						SO_REQ_SEP	},
-		{ OPT_CONFDIR,					(char*)"--confdir",					SO_REQ_SEP	},
-		{ OPT_CONFDIR,					(char*)"-cd",						SO_REQ_SEP	},
+		{ OPT_DATA_DIR,					(char*)"--data-dir",				SO_REQ_SEP	},
+		{ OPT_DATA_DIR,					(char*)"-dd",						SO_REQ_SEP	},
+		{ OPT_CONF_DIR,					(char*)"--conf-dir",				SO_REQ_SEP	},
+		{ OPT_CONF_DIR,					(char*)"-cd",						SO_REQ_SEP	},
 		{ OPT_GENERATOR_HEIGHT_MAP,		(char*)"--generator-height-map",	SO_REQ_SEP	},
 		{ OPT_GENERATOR_SEED,			(char*)"--generator-seed",			SO_REQ_SEP	},
 		{ OPT_GENERATOR_MAP,			(char*)"--generator-map",			SO_REQ_SEP	},
@@ -476,46 +479,44 @@ void parseArg(int argc, char *argv[])
 	while (args.Next()) {
 		switch (args.LastError()) {
 		case SO_OPT_INVALID:
-			cout << "<OPTION> " << args.OptionText() << " unrecognized" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " unrecognized" );
 			break;
 		case SO_OPT_MULTIPLE:
-			cout << "<OPTION> " << args.OptionText() << " matched multiple options" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " matched multiple options" );
 			break;
 		case SO_ARG_INVALID:
-			cout << "<OPTION> " << args.OptionText() << " does not accept any argument" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " does not accept any argument" );
 			break;
 		case SO_ARG_INVALID_TYPE:
-			cout << "<OPTION> " << args.OptionText() << " has an invalid argument format" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " has an invalid argument format" );
 			break;
 		case SO_ARG_MISSING:
-			cout << "<OPTION> " << args.OptionText() << " requires an argument" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " requires an argument" );
 			break;
 		case SO_ARG_INVALID_DATA:
-			cout << "<OPTION> " << args.OptionText() << " has an invalid argument data" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " has an invalid argument data" );
 			break;
 		case SO_SUCCESS:
-			cout << "<OPTION> " << args.OptionText() << " detected" << endl;
+			OPENCITY_OPTION( "" << args.OptionText() << " detected" );
 			break;
 		} // switch (args.LastError())
 
 		// Exit the program on error
 		if (args.LastError() != SO_SUCCESS) {
-			cout << "Try " << argv[0] << " --help for usage information" << endl;
-			exit( OC_ARGUMENT_ERROR );
+			OPENCITY_OPTION( "Try " << argv[0] << " --help for more usage information" );
+			exit( OC_ERROR_ARGUMENT );
 		}
 
 		switch (args.OptionId()) {
 		case OPT_GL_VERSION:
-			if (gVars.gpVideoSrf == NULL) {
-				(void)initSDL();
-			}
-			cout << "GL Vendor: " << glGetString( GL_VENDOR ) << endl;
-			cout << "GL Renderer: " << glGetString( GL_RENDERER ) << endl;
-			cout << "GL Version: " << glGetString( GL_VERSION ) << endl;
-			cout << "GL Extensions: " << glGetString( GL_EXTENSIONS ) << endl;
+			bGLVersion = true;
 			break;
 
-		case OPT_FULLSCREEN:
+		case OPT_GL_DRIVER:
+			gVars.gsOpenGLDriver = args.OptionArg();
+			break;
+
+		case OPT_FULL_SCREEN:
 			gVars.gboolFullScreen = true;
 			break;
 
@@ -523,14 +524,14 @@ void parseArg(int argc, char *argv[])
 			gVars.gboolUseAudio = false;
 			break;
 
-		case OPT_DATADIR:
+		case OPT_DATA_DIR:
 			sDataDir = formatPath(args.OptionArg());
-			cout << "<OPTION> DataDir is: \"" << sDataDir << "\"" << endl;
+			OPENCITY_OPTION( "The data directory is: \"" << sDataDir << "\"" );
 			break;
 
-		case OPT_CONFDIR:
+		case OPT_CONF_DIR:
 			sConfigDir = formatPath(args.OptionArg());
-			cout << "<OPTION> ConfDir is: \"" << sConfigDir << "\"" << endl;
+			OPENCITY_OPTION( "The configuration directory is: \"" << sConfigDir << "\"" );
 			break;
 
 		case OPT_GENERATOR_HEIGHT_MAP:
@@ -598,8 +599,8 @@ void parseArg(int argc, char *argv[])
 
 		case OPT_HELP:
 			cout << "Usage: " << argv[0]
-				<< " [-fs|--fullscreen] [-glv|--gl-version]"
-				<< " [-dd|--datadir newDataPath] [-cd|--confdir newConfigPath]"
+				<< " [-fs|--full-screen] [-glv|--gl-version] [-gld|--gl-driver openGLDriverName]"
+				<< " [-dd|--data-dir newDataPath] [-cd|--conf-dir newConfigPath]"
 				<< " [-na|--no-audio] [--generator-height-map heightMapPicture |"
 				<< " (--generator-seed seed [--generator-map MAP-TYPE] [--generator-water WATER-TYPE]"
 				<< " [--generator-map-shape MAP-SHAPE-TYPE] [--generator-tree-density TREE-DENSITY-TYPE])]"
@@ -612,17 +613,19 @@ void parseArg(int argc, char *argv[])
 				<< endl;
 			cout << "Warning: the command option overwrite the config file settings."
 				<< endl;
-			exit( OC_ARGUMENT_ERROR );
+			exit( OC_ERROR_ARGUMENT );
 			break;
 		}	// switch (args.OptionId())
 	} // while (args.Next())
+
+	return 0;
 }
 
 
    /*=====================================================================*/
-void displaySplash()
+static void displaySplash()
 {
-	#define OC_SPLASH_CLEAR_COLOR		.15, .15, .3, 1.0
+	#define OC_SPLASH_CLEAR_COLOR		.15, .4, .25, 1.0
 
 	glClearColor( OC_SPLASH_CLEAR_COLOR );
 	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
@@ -633,6 +636,8 @@ void displaySplash()
    /*=====================================================================*/
 static void displayStatus( const string & str )
 {
+	assert( gVars.gpRenderer != NULL );
+
 	uint x, y;
 
 	displaySplash();
@@ -648,24 +653,41 @@ static void displayStatus( const string & str )
    /*=====================================================================*/
 static int clientMode()
 {
-	int errCode = 0;
-
 // Initialize SDL
-	errCode = initSDL();
-	if (errCode != 0) {
-		return errCode;
+	if (gVars.gpVideoSrf == NULL) {
+		int errorCode = initSDL();
+		if (errorCode != 0) {
+			return errorCode;
+		}
 	}
 
 
-// set the window's caption
-	SDL_WM_SetCaption( PACKAGE " " VERSION, NULL );
-	SDL_WM_SetIcon( IMG_Load(ocDataDirPrefix("graphism/icon/OpenCity32.png").c_str()), NULL );
-
-
 // Create the mutex first
 	gVars.gpmutexSim = SDL_CreateMutex();
 
 
+// Load OpenGL extensions
+	gVars.gpExtensionMgr = new ExtensionManager();
+	if (gVars.gpExtensionMgr == NULL or !gVars.gpExtensionMgr->Load()) {
+		OPENCITY_FATAL( "Failed to load OpenGL extensions" );
+		SDL_Quit();
+
+#ifdef __WIN32__
+		MessageBox(
+			NULL,
+			"Failed to load OpenGL extensions. Please start a new \
+thread about your errror in the forum at http://www.opencity.info. Don't \
+forget to include the output from stdout.txt and stderr.txt files. They're \
+located in OpenCity executable directory.",
+			"OpenGL extensions loading error",
+			MB_ICONERROR | MB_OK | MB_SETFOREGROUND | MB_APPLMODAL
+		);
+#endif
+
+		return OC_ERROR_SDL_OPENGL;
+	}
+
+
 // Create the global renderer in order to use its text rendering functions
 	gVars.gpRenderer = new Renderer( gVars.guiCityWidth, gVars.guiCityLength );
 
@@ -676,10 +698,10 @@ static int clientMode()
 
 	if ( gVars.gpAudioMgr == NULL ) {
 		OPENCITY_FATAL( "Error while creating the audio manager" );
-		return OC_OUT_OF_MEMORY;
+		return OC_ERROR_MEMORY;
 	} else
 	if ( (gVars.gboolUseAudio == true)
-	  && (gVars.gpAudioMgr->OpenAudio() != OC_ERR_FREE))
+	  and (gVars.gpAudioMgr->OpenAudio() != OC_ERR_FREE))
 	{
 	// try to open the audio device IF we have successfully created an audio object
 		OPENCITY_INFO( "Audio open error ! OpenCity continues happily." );
@@ -726,7 +748,7 @@ static int clientMode()
 	City* pNewCity = new City( gVars.guiCityWidth, gVars.guiCityLength );
 	if (pNewCity == NULL) {
 		OPENCITY_FATAL( "Error while creating new city" );
-		return OC_OUT_OF_MEMORY;
+		return OC_ERROR_MEMORY;
 	}
 	displayStatus( "Almost done...");
 
@@ -750,6 +772,12 @@ static int clientMode()
 	// Restart with a new city from scratch
 	// WARNING: code duplication
 		if (bRestart) {
+		// Lock the simulator
+			SDL_LockMutex( gVars.gpmutexSim );
+
+		// Remove all moving objects
+			gVars.gpMoveMgr->Remove();
+
 			delete pNewCity;
 			delete gVars.gpMapMgr;
 			delete gVars.gpMapMaker;
@@ -768,11 +796,14 @@ static int clientMode()
 			pNewCity = new City( gVars.guiCityWidth, gVars.guiCityLength );
 			if (pNewCity == NULL) {
 				OPENCITY_FATAL( "Error while creating new city" );
-				return OC_OUT_OF_MEMORY;
+				return OC_ERROR_MEMORY;
 			}
 			gVars.gpRenderer->bHeightChange = true;
 			gVars.gpRenderer->bMinimapChange = true;
 
+		// Unlock the simulator
+			SDL_UnlockMutex( gVars.gpmutexSim );
+
 			bRestart = false;
 		}
 
@@ -824,6 +855,7 @@ static int clientMode()
 	delete gVars.gpAudioMgr;
 
 	delete gVars.gpRenderer;
+	delete gVars.gpExtensionMgr;
 
 // delete the simulators' mutex now
 	SDL_DestroyMutex( gVars.gpmutexSim );
@@ -832,17 +864,18 @@ static int clientMode()
 	gVars.gpVideoSrf = NULL;
 
 	SDL_Quit();					// WARNING: Calls free() on an invalid pointer. Detected by glibc
-	return errCode;
+
+	return 0;
 }
 
 
    /*=====================================================================*/
-void printCopyright() {
+static void printCopyright() {
 // Output the copyright text
 	cout << "Welcome to " << PACKAGE << " version " << VERSION << endl;
-	cout << "Copyright (C) by Duong-Khang NGUYEN. All rights reserved." << endl;
+	cout << "Copyright (C) by Duong Khang NGUYEN. All rights reserved." << endl;
 	cout << "   web  : http://www.opencity.info" << endl;
-	cout << "   email: neoneurone @ users sf net" << endl << endl;
+	cout << "   email: neoneurone @ gmail com" << endl << endl;
 
 	cout << "This program is released under the terms of" << endl;
 	cout << "GNU General Public License (See the COPYING file for more details)" << endl << endl;
@@ -856,7 +889,7 @@ void printCopyright() {
 	\return The pointer to the absolute directory. The caller must free
 the pointer if it's not used anymore.
 */
-char* findSaveDir()
+static char* findSaveDir()
 {
 	char* ret = NULL;
 
@@ -875,7 +908,7 @@ char* findSaveDir()
 // dll: shell32.dll
 
 	TCHAR szPath[MAX_PATH];
-	
+
 	if(SUCCEEDED(
 		SHGetFolderPath(NULL, CSIDL_APPDATA|CSIDL_FLAG_CREATE, NULL, 0, szPath)
 		)) {
@@ -898,7 +931,7 @@ char* findSaveDir()
 /** Try to detect and set the datadir, the confdir and the savedir using
 BinReloc library and win32 standard function
 */
-void detectProgramPath()
+static void detectProgramPath()
 {
 	char* pTemp = NULL;
 	BrInitError brError;
@@ -958,7 +991,7 @@ void detectProgramPath()
 #else
 	// Win32 uses \ as directory separtor
 		sSaveDir += "\\opencity\\";
-        CreateDirectory( sSaveDir.c_str(), NULL );		
+        CreateDirectory( sSaveDir.c_str(), NULL );
     // Replace \ by /
 	    string::size_type pos;
 	    while ( (pos = sSaveDir.find( '\\' )) != sSaveDir.npos ) {
@@ -978,10 +1011,10 @@ void detectProgramPath()
 /** Read the OpenCity's main settings file "opencity.xml"
 	\return "" if OK, otherwise the error description
 		0: if OK
-		OC_CONFIG_NOT_FOUND: the config file has not been found
-		OC_CONFIG_PARSE_ERROR: the was a parse error
+		OC_ERROR_NOT_FOUND: the config file has not been found
+		OC_ERROR_PARSE_CONFIG: there was a parse error
 */
-string readSettings()
+static string readSettings()
 {
 	string errorString = "";
 	TiXmlDocument settings;
@@ -1084,7 +1117,7 @@ string readSettings()
 
 
    /*=====================================================================*/
-void initGlobalVar()
+static void initGlobalVar()
 {
 // Config file and command line options
 	gVars.gboolUseAudio				= true;
@@ -1096,6 +1129,7 @@ void initGlobalVar()
 	gVars.guiScreenWidth			= OC_WINDOW_WIDTH;
 	gVars.guiScreenHeight			= OC_WINDOW_HEIGHT;
 	gVars.guiVideoBpp				= OC_WINDOW_BPP_DEFAULT;
+	gVars.gsOpenGLDriver			= "";
 
 	gVars.gsGeneratorHeightMap			= "";
 	gVars.guiGeneratorSeed				= time(NULL);
@@ -1136,9 +1170,9 @@ void initGlobalVar()
 
 
    /*=====================================================================*/
-#ifdef __WIN32__
-extern "C"
-#endif
+//#ifdef __WIN32__
+//extern "C"
+//#endif
 int main(int argc, char *argv[])
 {
 // Initialize the global settings variable to the default values
@@ -1148,7 +1182,10 @@ int main(int argc, char *argv[])
 	printCopyright();
 
 // Parse the command-line options
-	parseArg( argc, argv );
+	int returnCode = parseArg( argc, argv );
+	if (returnCode != 0) {
+		return returnCode;
+	}
 
 // Detect the main path: sDataDir and sSaveDir
 	detectProgramPath();
@@ -1157,25 +1194,28 @@ int main(int argc, char *argv[])
 	string errorDesc = readSettings();
 	if (errorDesc != "") {
 		OPENCITY_FATAL(
-			"The was an error while loading the settings file: \"" << errorDesc << "\"" << endl
+			"There was an error while loading the settings file: \"" << errorDesc << "\"" << endl
 			<< "If the main config file \"" << OC_CONFIG_FILE_FILENAME << "\" has not been found then" << endl
-			<< "try to specify the data directory with ""--datadir"" "
-			<< "and the configuration directory with ""--confdir""." << endl
+			<< "try to specify the data directory with ""--data-dir"" "
+			<< "and the configuration directory with ""--conf-dir""." << endl
 			<< "For example:" << endl
-			<< "    " << argv[0] << " --datadir \"/absolute/path/to/opencity/data\" "
-			<< "--confdir \"/absolute/path/to/opencity/conf\"" << endl
+			<< "    " << argv[0] << " --data-dir \"/absolute/path/to/opencity/data\" "
+			<< "--conf-dir \"/absolute/path/to/opencity/conf\"" << endl
 			<< "or" << endl
-			<< "    " << argv[0] << " --datadir \"../relative/path/to/opencity/data\" "
-			<< "--confdir \"../relative/path/to/opencity/conf\"" << endl
+			<< "    " << argv[0] << " --data-dir \"../relative/path/to/opencity/data\" "
+			<< "--conf-dir \"../relative/path/to/opencity/conf\"" << endl
 		);
-		exit(OC_CONFIG_NOT_FOUND);
+		exit(OC_ERROR_NOT_FOUND);
 	}
 
 // Test XPath
-//	PropertyManager2* pPropertyMgr = new PropertyManager2();
-//	delete pPropertyMgr;
-//	abort();
-
+// 	PropertyManager2* pPropertyMgr = new PropertyManager2();
+// 	const Property* myProperty = pPropertyMgr->Get( 1 );
+// 	OPENCITY_DEBUG( "Build cost 1: " << myProperty->uiDestroyCost );
+// 	const Property* myProperty2 = pPropertyMgr->Get( "graphism/residential/little_house/little_house.ac" );
+// 	OPENCITY_DEBUG( "Build cost 2: " << myProperty2->uiIncome );
+// 	delete pPropertyMgr;
+// 	abort();
 
 // Initialization of global variables
 	uipCurrentUI = NULL;
@@ -1185,7 +1225,7 @@ int main(int argc, char *argv[])
 	srand( time(NULL) );
 
 // Launch the game
-	int returnCode = clientMode();
+	returnCode = clientMode();
 
 	return returnCode;
 }
@@ -1220,7 +1260,7 @@ ocSaveDirPrefix( const string& s )
    /*=====================================================================*/
 string ocStrVersion()
 {
-	ostringstream oss;
+	std::ostringstream oss;
 
 	oss << OC_VERSION << "." << OC_PATCHLEVEL << "." << OC_SUBLEVEL;
 	return oss.str();
@@ -1235,27 +1275,3 @@ long ocLongVersion()
 	lVersion = OC_VERSION*65536 + OC_PATCHLEVEL*256 + OC_SUBLEVEL;
 	return lVersion;
 }
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
